#version 330 core

in vec2 vUV;
in vec2 vFragScreenPos;
out vec4 FragColor;

uniform sampler2D uTex;
uniform sampler2D uNormalMap;
uniform float uRotation;

uniform float uBrightness;
uniform float uSaturation;
uniform float uGamma;

#define MAX_LIGHTS 512

uniform int   uLightCount;
uniform vec2  uLightPos[MAX_LIGHTS];
uniform vec3  uLightColor[MAX_LIGHTS];
uniform float uLightIntensity[MAX_LIGHTS];
uniform float uLightRadius[MAX_LIGHTS];

void main()
{
    // Rotate UV 90° clockwise
    vec2 rotatedUV = vec2(vUV.y, 1.0 - vUV.x);

    vec4 texColor = texture(uTex, rotatedUV);
    if (texColor.a < 0.1)
        discard;

    vec3 n = texture(uNormalMap, rotatedUV).rgb * 2.0 - 1.0;
    n.y = -n.y;

    float c = cos(uRotation);
    float s = sin(uRotation);
    mat2 rot = mat2(c, -s, s, c);
    n.xy = rot * n.xy;

    vec3 normal = normalize(n);
    vec3 finalLight = vec3(0.0);

    for (int i = 0; i < uLightCount; ++i)
    {
        vec2 lightVec = uLightPos[i] - vFragScreenPos;
        float dist = length(lightVec);

        if (dist < uLightRadius[i])
        {
            vec2 lightDir2D = normalize(lightVec);
            vec3 lightDir = normalize(vec3(lightDir2D, 1.0));
            float attenuation = smoothstep(uLightRadius[i], 0.0, dist);
            float diff = max(dot(normal, lightDir), 0.0);
            vec3 light = uLightColor[i] * diff * attenuation * uLightIntensity[i];
            finalLight += light;
        }
    }

    vec3 result = texColor.rgb * finalLight;

    result *= uBrightness;

    float gray = dot(result, vec3(0.299, 0.587, 0.114));
    result = mix(vec3(gray), result, uSaturation);

    result = pow(result, vec3(1.0 / uGamma));

    result = clamp(result, 0.0, 1.0);

    FragColor = vec4(result, texColor.a);
}
